@@ -43,6 +43,7 @@ from .errors import MissingDirError, MissingFileError, BuilderError, \
 from .ipaddresses import IPNetwork, IPAddress
 from .irrdb import IRRDBInfo
 from .cached_objects import CachedObject, normalize_expiry_time
+from .reject_reasons import REJECT_REASONS
 
 
 class ConfigBuilder(object):
@@ -58,24 +59,6 @@ class ConfigBuilder(object):
 
     IGNORABLE_ISSUES = []
 
-    REJECT_REASONS = {
-        "1": "Invalid AS_PATH length",
-        "2": "Prefix is bogon",
-        "3": "Prefix is in global blacklist",
-        "4": "Invalid AFI",
-        "5": "Invalid NEXT_HOP",
-        "6": "Invalid left-most ASN",
-        "7": "Invalid ASN in AS_PATH",
-        "8": "Transit-free ASN in AS_PATH",
-        "9": "Origin ASN not in IRRDB AS-SETs",
-        "10": "IPv6 prefix not in global unicast space",
-        "11": "Prefix is in client blacklist",
-        "12": "Prefix not in IRRDB AS-SETs",
-        "13": "Invalid prefix length",
-        "14": "RPKI INVALID route",
-        "15": "Never via route-servers ASN in AS_PATH",
-    }
-
     def validate_bgpspeaker_specific_configuration(self):
         """Check compatibility between config and target BGP speaker
 
@@ -143,6 +126,7 @@ class ConfigBuilder(object):
                  cache_dir=None, cache_expiry=CachedObject.DEFAULT_EXPIRY,
                  bgpq3_path="bgpq4", bgpq3_host=IRRDBInfo.BGPQ3_DEFAULT_HOST,
                  bgpq3_sources=IRRDBInfo.BGPQ3_DEFAULT_SOURCES,
+                 bgpq3_timeout=IRRDBInfo.BGPQ3_DEFAULT_TIMEOUT,
                  rtt_getter_path=None, threads=4,
                  ip_ver=None, perform_graceful_shutdown=False,
                  ignore_errors=[], live_tests=False,
@@ -310,8 +294,11 @@ class ConfigBuilder(object):
 
                 - *bgpq3_path* program's configuration file option.
 
-            bgpq3_host (str): the host that will be queried by bgpq3/bgpq4; this
-                will be used to set the *-h* argument of the program.
+            bgpq3_host (str): the host(s) that will be queried by bgpq3/bgpq4; this
+                will be used to set the *-h* argument of the program. Multiple hosts
+                can be passed using a comma-delimited string, in which case they
+                will be used sequentially in case of failures of the IRR queries or
+                timeouts.
 
                 Same of:
 
@@ -325,6 +312,12 @@ class ConfigBuilder(object):
 
                 - *bgpq3_sources* program's configuration file option.
 
+            bgpq3_timeout (int): timeout for the bgpq4/bgpq3 queries (in seconds).
+
+                Same of:
+
+                - *bgpq3_timeout* program's configuration file option.
+
             rtt_getter_path (str): path to the program that is executed to
                 determine the RTT of a peer.
                 Syntax and details can be found at the following URL:
@@ -371,6 +364,7 @@ class ConfigBuilder(object):
         self.bgpq3_path = bgpq3_path
         self.bgpq3_host = bgpq3_host
         self.bgpq3_sources = bgpq3_sources
+        self.bgpq3_timeout = bgpq3_timeout
 
         self.rtt_getter_path = rtt_getter_path
 
@@ -607,7 +601,8 @@ class ConfigBuilder(object):
         self.data["rtt_based_functions_are_used"] = \
             self.cfg_general.rtt_based_functions_are_used
         self.data["perform_graceful_shutdown"] = self.perform_graceful_shutdown
-        self.data["reject_reasons"] = self.REJECT_REASONS
+        self.data["reject_reasons"] = REJECT_REASONS
+        self.data["any_reject_cause_map_community_set"] = self.cfg_general.any_reject_cause_map_community_set
 
         def ipaddr_ver(ip):
             return IPNetwork(ip).version
@@ -880,7 +875,7 @@ class OpenBGPDConfigBuilder(ConfigBuilder):
     LOCAL_FILES_BASE_DIR = "/etc/bgpd"
 
     AVAILABLE_VERSION = ["6.0", "6.1", "6.2", "6.3", "6.4", "6.5", "6.6", "6.7",
-                         "6.8", "6.9", "7.0", "7.1"]
+                         "6.8", "6.9", "7.0", "7.1", "7.2"]
     DEFAULT_VERSION = AVAILABLE_VERSION[-1]
 
     IGNORABLE_ISSUES = ["path_hiding", "transit_free_action",
